#include "common.h"

/* 
   mkover - maintain the overview database
   Olaf Titz, Aug. 95. Public domain.
*/

static void usage(const char *n)
{
  fprintf(stderr, "Usage: %s [options] [group...]\n", n);
  fprintf(stderr, "  -e: expiration checking\n");
  fprintf(stderr, "  -p: only expire\n");
  fprintf(stderr, "  -v: verbose\n");
  exit(127);
}

int tot_grp=0, tot_add=0, tot_exp=0;

static int expire=0, addnew=1;
int verbose=0;
static int daem=0;

#ifdef DEBUG
extern int malloc_usage(void);

static void testmode(char *argv[])
{
  int mi, ma, i, me;
  char *c;
  ++verbose;
  me=malloc_usage();
  mi=atoi(argv[optind++]);
  ma=atoi(argv[optind]);
  for(i=mi; i<=ma; i++) {
    if ((c=mkover(i))) {
      fputs(c, stdout);
      fputc('\n', stdout);
      free(c);
    }
  }
  printf("malloc: %d, %d\n", me, malloc_usage());
  exit(0);
}
#endif

void main(int argc, char *argv[])
{
  int i;
  FILE *a;
  char gr[128], fl[128];
  int mi, ma;
#ifdef DEBUG
  int me;
#endif

  while ((i=getopt(argc, argv, "d:epvt"))>0) {
    switch(i) {
    case 'd': daem=atoi(optarg);
      fprintf(stderr, "warning: daemon mode not implemented\n");
      break;
    case 'e': ++expire; break;
    case 'p': ++expire; addnew=0; break;
    case 'v': ++verbose; break;
#ifdef DEBUG
    case 't': testmode(argv); break;
#endif
    default: usage(argv[0]);
    }
  }
  {
    char buf[PATH_MAX];
    sprintf(buf, "%s/active", NEWSLIB);
    if (!(a=fopen(buf, "r"))) {
      fprintf(stderr, "Can't open active: %s\n", strerror(errno));
      exit(1);
    }
  }
  i=lock();
  if (i>0) {
    fprintf(stderr, "Locked by process %d\n", i);
    exit(2);
  }
  if (i<0) {
    fprintf(stderr, "Can't lock: %s\n", strerror(errno));
    exit(3);
  }
  setsig(NULL);
  setbuf(stdout, NULL);
#ifdef DEBUG
  me=malloc_usage();
#endif
  while (fscanf(a, "%s %d %d %s\n", gr, &ma, &mi, fl)>0) {
    if ((optind==argc) || (matches(gr, argv, optind))) {
      if (fl[0]=='y' || fl[0]=='m' || fl[0]=='n') {
	if (verbose)
	  printf("%s ", gr);
	i = dogroup(gr, mi, ma, expire, addnew);
	putchar(i);
	if (verbose)
	  putchar('\n');
	++tot_grp;
      } else {
	putchar('x');
      }
    }
  }
#ifdef DEBUG
  if (verbose)
    printf("malloc: %d, %d\n", me, malloc_usage());
#endif
  fclose(a);
  finish(0);
}
