#define I_MMAN
#include "common.h"

/* Produce one article's overview line */
/* Returns a malloc'ed string */

char *
mkover(int art)
{
	int f;
	char bn[NUMLEN];
#ifndef USE_MMAP
	int rpos = 0, apos = 0;
	int bufs = CHUNK, mrd = CHUNK - 1, nrd;
#else
	int lines = 0;
	char lineb[NUMLEN];
#endif
	char *buf, *n = NULL;
	char *p, *q, *r, c;
	int l, i;
	struct stat st;
	int overrec[OV_XREF + 1];
	int overlen[OV_XREF + 1] = {0, 0, 0, 0, 0, 0, 0, 0};

#define err_c  { close(f); return NULL; }
#define err_f  { goto fin; }
#define err_cf { close(f); goto fin; }

	sprintf(bn, "%d", art);
	/*
	printf("mkover(%d)", art);
	fflush(stdout);
	*/
	if ((f = open(bn, O_RDONLY)) < 0)
	{
		/*
		printf("Article not found\n");
		fflush(stdout);
		*/
		return NULL;
	}
	if ((fstat(f, &st) < 0) || (!(S_ISREG(st.st_mode))))
	{
		/*
		printf("Stat failed\n");
		fflush(stdout);
		*/
		err_c;
	}

#ifdef USE_MMAP
	if ((buf = mmap(NULL, st.st_size, PROT_READ, MAP_FILE | MAP_PRIVATE,
					f, 0)) == (char *) -1)
	{
		/*
		printf("mmap failed\n");
		fflush(stdout);
		*/
		err_c;
	}
	p = buf;
#else
	if (!(buf = malloc(CHUNK)))
		err_c;
#endif

	/* read & analyze header */

#ifndef USE_MMAP
	while (1)
	{
		if ((nrd = read(f, buf + rpos, mrd)) <= 0)
			err_cf;
		buf[rpos + nrd] = '\0';
		p = buf + apos;
#endif

		r = p;
		/*
		printf("Reading header lines, found: ");
		fflush(stdout);
		*/
		/* Read lines and continuation lines */
		while ((q = strchr(p, '\n')))
		{
			if (q[1] == ' ' || q[1] == '\t')
			{
				p = q + 1;
				continue;
			}
			q++;

#define grok(s,l,n,f) \
      if (!strncasecmp(r,(s),(l))) \
           { /* printf("%s ", (s)); fflush(stdout); */ overrec[(f)]=r+(n)-buf; overlen[(f)]=q-r-(n); }

			/*
			 * Match without whitespace, because some posters are broken,
			 * using tabs
			 */
			grok("Subject: ", 8, 9, OV_SUBJECT);
			grok("From: ", 5, 6, OV_FROM);
			grok("Date: ", 5, 6, OV_DATE);
			grok("Message-ID: ", 11, 12, OV_MESSAGEID);
			grok("References: ", 11, 12, OV_REFERENCES);
			grok("Lines: ", 6, 7, OV_LINES);
			grok("Xref: ", 5, 0, OV_XREF);
			/* Empty line is end of header */
			if (*q == '\n')
				goto eoh;
			r = p = q;
		}

#ifndef USE_MMAP
		apos = p - buf;
		rpos += nrd;
		if ((mrd -= nrd) <= 0)
		{
			bufs += CHUNK;
			if (!(buf = realloc(buf, bufs)))
			{
				close(f);
				return NULL;
			}
			mrd = CHUNK;
		}
	}
#endif

  eoh:
	/*
	printf("done\n");
	fflush(stdout);
	*/
	close(f);
#ifdef USE_MMAP
	/* If lines not found, count them, but only on mmap version */
	if (!overlen[OV_LINES])
	{
		p = q + 1;
		while ((p = strchr(p, '\n')))
		{
			lines++;
			p++;
		}
		sprintf(lineb, "%d", lines);
		/*
		 * printf("Lines counted in article %d is %s\n", art, lineb);
		 * fflush(stdout);
		 */
#ifndef NO_POINTER_ARITH
		overrec[OV_LINES] = lineb - buf;
#endif
		overlen[OV_LINES] = strlen(lineb);
	}
#endif

	/* Article length is file size less header size */
	sprintf(bn, "%ld\n", st.st_size - (q - buf));
#ifndef NO_POINTER_ARITH
	overrec[OV_BYTES] = bn - buf;
#endif
	overlen[OV_BYTES] = strlen(bn);

	for (l = 0, i = 0; i <= OV_XREF; ++i)
		l += overlen[i];
	if (!(n = malloc(NUMLEN + l + OV_XREF + 3)))	/* leave space for artno
													 * & tabs */
		err_f;
	sprintf(n, "%d\t", art);
	p = n + strlen(n);
	/*
	fprintf(stdout, "Article %d\n", art);
	fflush(stdout);
	*/
	for (i = 0; i <= OV_XREF; ++i)
	{
		/*
		fprintf(stdout, "%d: ", i);
		*/
		if (overlen[i])
		{
#ifdef NO_POINTER_ARITH
			if (i == OV_BYTES)
				q = bn;
			else if (i == OV_LINES)
				q = lineb;
			else
#endif
				q = buf + overrec[i];
			l = overlen[i];
			while (l--)
			{
				if ((c = *q++) == '\n')
					continue;
				*p++ = (c < ' ') ? ' ' : c;
				/*
				fputc(*(p - 1), stdout);
				fflush(stdout);
				*/
			}
		}
		/*
		fputc('\n', stdout);
		fflush(stdout);
		*/
		*p++ = '\t';
	}
	--p;
	if (!overlen[OV_XREF])
		--p;
	*p = '\0';

  fin:
#ifdef USE_MMAP
	munmap(buf, st.st_size);
#else
	free(buf);
#endif
	/*
	fprintf(stdout, "Returning: %s\n", n);
	fflush(stdout);
	*/
	return n;
}
