#include "common.h"

/* Automagic XOVER support for nntpd: read the overview file, generate
   missing entries, update the file as entries are requested. Adjusts
   itself to file availability and permission settings. Uses the "step
   forward" algorithm as recommended by Davison. Big win for trn :-)

   -ot.1995-08-27 */

static FILE *overfile=NULL;
static int overpos;
static int overmin=0;
static int maywrite;
static char *buf=NULL;
static int bufl=CHUNK;

/* Done with a group. */
void xo_close(void)
{
  fclose(overfile);
  overfile=NULL;
  unlock();
}

/* Step into a group. Returns 1 if successful, 0 if not. */
int xo_group(char *gr, int mi)
{
  char b[PATH_MAX];
  char *p;
  char c;

  if (overfile)
    xo_close();
  strcpy(b, NEWSARTS);
  p=b+strlen(b);
  *p++='/';
  do {
    c=*gr++;
    if (c=='.')
      c='/';
    *p++=c;
  } while (c);
  if (chdir(b)<0)
    return 0;
  if (!buf) {
    if (!(buf=malloc(bufl)))
      return 0;
  }
  overmin=--mi;
  maywrite=1;
  if (lock()==0) {
    if ((overfile=fopen(".overview", "a+"))) {
      rewind(overfile);
      overpos=0;
    } else if (errno==EACCES) {
      maywrite=0;
      overfile=fopen(".overview", "r");
    }
  }
#ifdef DEBUG
  if (verbose)
    fprintf(stderr, "%c%c ", (overfile)?'R':'-', 
	    (overfile && maywrite)?'W':'-');
#endif
  return 1;
}

/* Give overview line for one article, after xo_group was called.
   Returns a malloc'ed string */
char *xo_xover(int n)
{
  char *b=NULL;
  int l;

  if (!overfile)
    return mkover(n);

  if (overpos>n) {
#ifdef DEBUG
    if (verbose)
      fprintf(stderr, ".0 ");
#endif
    rewind(overfile);
    overpos=0;
  }
  assert(buf);

  /* Read the overview file... */
  while ((overpos<n) && !feof(overfile)) {
    l=0;
    while ((b=fgets(buf+l, bufl-l, overfile)) &&
	   (l=strlen(buf),
	    buf[l-1]!='\n')) {
      bufl+=CHUNK;
      if (!(buf=realloc(buf, bufl))) {
	xo_close(); return NULL;
      }
    }
    if (!b) 
      break;
    overpos=atoi(buf);
#ifdef DEBUG
    if (verbose)
      fprintf(stderr, ".%d ", overpos);
#endif
  }

  /* ...found it there? */
  if (overpos==n) {
#ifdef DEBUG
    if (verbose)
      fprintf(stderr, ":%d ", overpos);
#endif
    return strdup(buf);
  }

  /* Can we update the file? */
  if (!maywrite) {
#ifdef DEBUG
    if (verbose)
      fprintf(stderr, "-%d ", n);
#endif
    return mkover(n);
  }

  if (!overpos)
    overpos=overmin;
  /* Update the overview file up to the requested entry. */
  while (overpos<n) {
    ++overpos;
    b=mkover(overpos);
    if (b) {
#ifdef DEBUG
      if (verbose)
	fprintf(stderr, "+%d ", overpos);
#endif
      fputs(b, overfile);
      fputc('\n', overfile);
      if (overpos==n) {
	if ((l=strlen(b))>=bufl) {
	  do {
	    bufl+=CHUNK;
	  } while (l>=bufl);
	  if (!(buf=realloc(buf, bufl))) {
	    free(b); xo_close(); return NULL;
	  }
	}
	strcpy(buf, b);
	return b;
      }
      free(b);
    }
  }
  return NULL;
}
